package at.ac.tuwien.dbai.staff.dvorak.alternation.compiler;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;

import org.antlr.runtime.ANTLRInputStream;
import org.antlr.runtime.RecognitionException;
import org.antlr.runtime.TokenRewriteStream;
import org.antlr.stringtemplate.StringTemplateGroup;

/**
 * Gernerates Java code from an alternating algorithm
 *
 * @author Wolfgang Dvorak
 *
 */
public class atmCompile {
	public static final String USAGE = "Usage: java atmCompile ATMFILE";
	/**
	 * Takes a .atm file  and computes a java class
	 *
	 * @param inputPath path to a .atm file
	 * @param outputPath path to save the output .java file
	 * @throws IOException
	 * @throws RecognitionException
	 */
	public static void compile (String inputPath, String outputPath) throws IOException, RecognitionException{

		StringTemplateGroup templates =
			new StringTemplateGroup(
					new BufferedReader(
							new InputStreamReader(
									ClassLoader.getSystemResourceAsStream("at/ac/tuwien/dbai/staff/dvorak/alternation/compiler/atm.stg"))));
		//groupFileR .close();



		ANTLRInputStream input = new ANTLRInputStream(new  FileInputStream(inputPath));
		alternating_machineLexer lexer = new alternating_machineLexer(input);
		TokenRewriteStream tokens = new TokenRewriteStream(lexer);
		alternating_machineParser parser= new alternating_machineParser(tokens);
		parser.setTemplateLib(templates);
		parser.atm_file();

	    FileWriter  writer = new FileWriter(outputPath);
	    BufferedWriter  bWriter = new BufferedWriter(writer);
	    bWriter.write(tokens.toString());
	    bWriter.close();
	    System.out.println(outputPath +" generated");
	}

	public static void main(String[] args) throws IOException, RecognitionException {
	
		if (args.length > 0) {
			if (args.length==1) {
				compile(args[0],args[0].substring(0,args[0].lastIndexOf(".atm"))+".java");
			} else {
				compile(args[0],args[1]);
			}
    	}
    	else {
			System.out.println(USAGE);
		}
	}
	
	public static void compileExamples() throws IOException, RecognitionException {
		String name="src/at/ac/tuwien/dbai/staff/dvorak/alternation/examples/GGAtm";
		compile(name+".atm",name+".java");
		name="src/at/ac/tuwien/dbai/staff/dvorak/alternation/examples/OneDimensionalCellularAutomata";
		compile(name+".atm",name+".java");
		name="src/at/ac/tuwien/dbai/staff/dvorak/alternation/examples/AlternatingTuringMachine";
		compile(name+".atm",name+".java");
		name="src/at/ac/tuwien/dbai/staff/dvorak/alternation/examples/Horn";
		compile(name+".atm",name+".java");
		name="src/at/ac/tuwien/dbai/staff/dvorak/alternation/examples/Horn2";
		compile(name+".atm",name+".java");
		name="src/at/ac/tuwien/dbai/staff/dvorak/alternation/examples/Horn3";
		compile(name+".atm",name+".java");
		name="src/at/ac/tuwien/dbai/staff/dvorak/alternation/examples/CatAndMouse";
		compile(name+".atm",name+".java");
		name="src/at/ac/tuwien/dbai/staff/dvorak/alternation/examples/CatAndMouse2";
		compile(name+".atm",name+".java");
		name="src/at/ac/tuwien/dbai/staff/dvorak/alternation/examples/CatAndMouse3";
		compile(name+".atm",name+".java");
		name="src/at/ac/tuwien/dbai/staff/dvorak/alternation/examples/LFMC";
		compile(name+".atm",name+".java");
		name="src/at/ac/tuwien/dbai/staff/dvorak/alternation/examples/MCVP";
		compile(name+".atm",name+".java");
		name="src/at/ac/tuwien/dbai/staff/dvorak/alternation/examples/PEBBLE";
		compile(name+".atm",name+".java");
		name="src/at/ac/tuwien/dbai/staff/dvorak/alternation/examples/PEBBLE2";
		compile(name+".atm",name+".java");
		name="src/at/ac/tuwien/dbai/staff/dvorak/alternation/examples/TicTacToe";
		compile(name+".atm",name+".java");
	}
}
