package at.ac.tuwien.dbai.staff.dvorak.alternation.examples;
import java.util.Set;
import java.util.Map;


public class AlternatingTuringMachine {
	private at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.ATM<Inputtape,Worktape> atm;

	public class Inputtape {
		char[] inputtape;
		TransitionFunction delta;
		Map<Integer,Character> stateType;
		int initialState;
		int C;

		private Inputtape(char[] inputtape,TransitionFunction delta,Map<Integer,Character> stateType,int initialState,int C) {
			super();
			this.inputtape=inputtape;
			this.delta=delta;
			this.stateType=stateType;
			this.initialState=initialState;
			this.C=C;
		}
	}

    public class Worktape implements at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.InterfaceWorktape<Inputtape> {
    	int activeState;
    	boolean first;
    	int inputCursor;
    	int workCursor;
    	char[] worktape;

    	private Worktape(){
    		super();
    	}

    	private Worktape(Worktape worktape) {
    		this.activeState = worktape.activeState;
    		this.first = worktape.first;
    		this.inputCursor = worktape.inputCursor;
    		this.workCursor = worktape.workCursor;
    		this.worktape = worktape.worktape.clone();
    	}

    	public void reset(Inputtape atmInputtape) {

    		this.worktape = new char[atmInputtape.C];
    		this.inputCursor = 0;
    		this.workCursor = 0;
    		this.activeState = atmInputtape.initialState;
    		this.first = true;
    	}

    	public Worktape clone(){
    		return new Worktape(this);
    	};

    	public boolean equals( Object o ) {
        	if ( o == null ) return false;
            if (! o.getClass().equals(getClass()) ) return false;
            Worktape tape2=(Worktape) o;
    		if(this.activeState!=tape2.activeState) return false;
    		if(this.first!=tape2.first) return false;
    		if(this.inputCursor!=tape2.inputCursor) return false;
    		if(this.workCursor!=tape2.workCursor) return false;
    		if(this.worktape==null){
    			if(!(tape2.worktape==null)) return false;
    		} else {
    		    if(!java.util.Arrays.equals(this.worktape,tape2.worktape)) return false;
    		}
            return true;
    	}

    	@Override
    	public int hashCode() {
    		final int prime = 31;
    		int result = 1;


    		result = prime * result +  ((Object)activeState).hashCode();
    		result = prime * result +  ((Object)first).hashCode();
    		result = prime * result +  ((Object)inputCursor).hashCode();
    		result = prime * result +  ((Object)workCursor).hashCode();
    		result =  prime * result + java.util.Arrays.hashCode(worktape);

    		return result;
    	}

    	public String toString() {
    		StringBuffer ausgabe= new StringBuffer();
    		 ausgabe.append("activeState=(" +((Object)activeState).toString()+"); ");
    		 ausgabe.append("first=(" +((Object)first).toString()+"); ");
    		 ausgabe.append("inputCursor=(" +((Object)inputCursor).toString()+"); ");
    		 ausgabe.append("workCursor=(" +((Object)workCursor).toString()+"); ");
    		return ausgabe.toString();
    	}
    }

    private class alternatingTuringMachine extends at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.State<Inputtape,Worktape>  {
    	
    	public at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.ResultTuple<Worktape> compute(Worktape atmWorktape) {
    		at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.ResultTuple<Worktape> atmResultTuple;
    		Worktape atmWorktape2;
    		if (atmWorktape.first) {
    					java.util.Arrays.fill(atmWorktape.worktape, 'b'); 
    					atmWorktape.first=false;	
    				}		
    				if(atmInputtape.stateType.get(atmWorktape.activeState).equals('A')) {
    					return at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.ResultTuple.ACCEPT;
    				}		
    				if(atmInputtape.stateType.get(atmWorktape.activeState).equals('R')) {
    					return at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.ResultTuple.REJECT;
    				}
    				if(atmInputtape.stateType.get(atmWorktape.activeState).equals('F')) {
    					atmResultTuple = new at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.ResultTuple<Worktape>(false, false, true);
    		atmWorktape2=atmWorktape.clone();
    		for (TransitionFunction.Output outputTuple : atmInputtape.delta.compute(atmInputtape.inputtape[atmWorktape.inputCursor],atmWorktape.worktape[atmWorktape.workCursor],atmWorktape.activeState)){
    							{
    								atmWorktape.worktape[atmWorktape.workCursor]=outputTuple.workSymbol;						
    								atmWorktape.activeState =outputTuple.newState;
    								atmWorktape.inputCursor+=outputTuple.inputCursor;
    								atmWorktape.workCursor+=outputTuple.workCursor;
    							}
    		atmResultTuple.add((byte)0,atmWorktape);
    		atmWorktape=atmWorktape2.clone();
    						}
    		return atmResultTuple;
    				}
    				if(atmInputtape.stateType.get(atmWorktape.activeState).equals('E')) {
    					atmResultTuple = new at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.ResultTuple<Worktape>(false, false, false);
    		atmWorktape2=atmWorktape.clone();
    		for (TransitionFunction.Output outputTuple : atmInputtape.delta.compute(atmInputtape.inputtape[atmWorktape.inputCursor],atmWorktape.worktape[atmWorktape.workCursor],atmWorktape.activeState)){
    							{
    								atmWorktape.worktape[atmWorktape.workCursor]=outputTuple.workSymbol;						
    								atmWorktape.activeState =outputTuple.newState;
    								atmWorktape.inputCursor+=outputTuple.inputCursor;
    								atmWorktape.workCursor+=outputTuple.workCursor;
    							}
    		atmResultTuple.add((byte)0,atmWorktape);
    		atmWorktape=atmWorktape2.clone();
    						}
    		return atmResultTuple;
    				}				
    				return at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.ResultTuple.REJECT;
    	}
    }

    public boolean compute(char[] inputtape,TransitionFunction delta,Map<Integer,Character> stateType,int initialState,int C) {

    	Inputtape atmInput = new Inputtape(inputtape,delta,stateType,initialState,C);
    	if (this.atm==null) {
    		Worktape atmWork = new  Worktape();
    		java.util.Map<Byte,at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.State<Inputtape,Worktape>> atmStates = new java.util.HashMap<Byte,at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.State<Inputtape,Worktape>>();

    		atmStates.put((byte)0,new alternatingTuringMachine());

    		this.atm = new at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.ATMacyclic<Inputtape,Worktape>(atmStates, atmWork, (byte)0);
        }
        return atm.compute(atmInput);
    }

	public at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.ComputationTree<Worktape> getComputationTree(){
		at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.ComputationTree<Worktape> cT=this.atm.getComputationTree();
		cT.setStateIdToName((byte)0, "alternatingTuringMachine");
		cT.setNameToStateId("alternatingTuringMachine",(byte) 0);

		return cT;
	}
}