package at.ac.tuwien.dbai.staff.dvorak.alternation.examples;


public class CatAndMouse2 {
	private at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.ATM<Inputtape,Worktape> atm;

	public class Inputtape {
		Graph<Integer> gameboard;
		Graph.Node<Integer> catStart;
		Graph.Node<Integer> mouseStart;
		Graph.Node<Integer> goal;

		private Inputtape(Graph<Integer> gameboard,Graph.Node<Integer> catStart,Graph.Node<Integer> mouseStart,Graph.Node<Integer> goal) {
			super();
			this.gameboard=gameboard;
			this.catStart=catStart;
			this.mouseStart=mouseStart;
			this.goal=goal;
		}
	}

    public class Worktape implements at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.InterfaceWorktape<Inputtape> {
    	Graph.Node<Integer> catPosition;
    	Graph.Node<Integer> mousePosition;

    	private Worktape(){
    		super();
    	}

    	private Worktape(Worktape worktape) {
    		this.catPosition = worktape.catPosition;
    		this.mousePosition = worktape.mousePosition;
    	}

    	public void reset(Inputtape atmInputtape) {

    		this.catPosition=null;
    		this.mousePosition=null;
    		this.catPosition = atmInputtape.catStart;
    		this.mousePosition = atmInputtape.mouseStart;
    	}

    	public Worktape clone(){
    		return new Worktape(this);
    	};

    	public boolean equals( Object o ) {
        	if ( o == null ) return false;
            if (! o.getClass().equals(getClass()) ) return false;
            Worktape tape2=(Worktape) o;
    		if(this.catPosition==null){
    			if(!(tape2.catPosition==null)) return false;
    		} else {
    		  	if(!this.catPosition.equals(tape2.catPosition)) return false;
    		}
    		if(this.mousePosition==null){
    			if(!(tape2.mousePosition==null)) return false;
    		} else {
    		  	if(!this.mousePosition.equals(tape2.mousePosition)) return false;
    		}
            return true;
    	}

    	@Override
    	public int hashCode() {
    		final int prime = 31;
    		int result = 1;


    		result = prime * result + ((catPosition == null) ? 0 : catPosition.hashCode());
    		result = prime * result + ((mousePosition == null) ? 0 : mousePosition.hashCode());

    		return result;
    	}

    	public String toString() {
    		StringBuffer ausgabe= new StringBuffer();

    		if (catPosition==null) {
    			ausgabe.append("catPosition=null; ");
    		} else {
    			ausgabe.append("catPosition=("+((Object)catPosition).toString()+"); ");
    		}


    		if (mousePosition==null) {
    			ausgabe.append("mousePosition=null; ");
    		} else {
    			ausgabe.append("mousePosition=("+((Object)mousePosition).toString()+"); ");
    		}

    		return ausgabe.toString();
    	}
    }

    private class mouseTurn extends at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.State<Inputtape,Worktape>  {
    	
    	public at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.ResultTuple<Worktape> compute(Worktape atmWorktape) {
    		at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.ResultTuple<Worktape> atmResultTuple;
    		Worktape atmWorktape2;
    		if (atmWorktape.mousePosition==atmWorktape.catPosition){
    					return at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.ResultTuple.REJECT;
    				}
    				atmResultTuple = new at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.ResultTuple<Worktape>(false, false, false);
    		atmWorktape2=atmWorktape.clone();
    		for(Graph.Node child: atmWorktape.mousePosition.getChildren()){
    						{
    							atmWorktape.mousePosition=child;
    						}
    		atmResultTuple.add((byte)1,atmWorktape);
    		atmWorktape=atmWorktape2.clone();
    					}
    		return atmResultTuple;
    	}
    }

    private class catTurn extends at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.State<Inputtape,Worktape>  {
    	
    	public at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.ResultTuple<Worktape> compute(Worktape atmWorktape) {
    		at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.ResultTuple<Worktape> atmResultTuple;
    		Worktape atmWorktape2;
    		if (atmWorktape.mousePosition==atmInputtape.goal){
    					return at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.ResultTuple.ACCEPT;
    				}
    				if (atmWorktape.mousePosition==atmWorktape.catPosition){
    					return at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.ResultTuple.REJECT;
    				}
    				atmResultTuple = new at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.ResultTuple<Worktape>(false, false, true);
    		atmWorktape2=atmWorktape.clone();
    		{
    					}
    		atmResultTuple.add((byte)0,atmWorktape);
    		atmWorktape=atmWorktape2.clone();
    					// cat move
    					for(Graph.Node child: atmWorktape.catPosition.getChildren()){
    						if(child!=atmInputtape.goal){
    							{
    								atmWorktape.catPosition=child;
    							}
    		atmResultTuple.add((byte)0,atmWorktape);
    		atmWorktape=atmWorktape2.clone();
    						}
    					}
    		return atmResultTuple;
    	}
    }

    public boolean compute(Graph<Integer> gameboard,Graph.Node<Integer> catStart,Graph.Node<Integer> mouseStart,Graph.Node<Integer> goal) {

    	Inputtape atmInput = new Inputtape(gameboard,catStart,mouseStart,goal);
    	if (this.atm==null) {
    		Worktape atmWork = new  Worktape();
    		java.util.Map<Byte,at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.State<Inputtape,Worktape>> atmStates = new java.util.HashMap<Byte,at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.State<Inputtape,Worktape>>();

    		atmStates.put((byte)1,new catTurn());
    		atmStates.put((byte)0,new mouseTurn());

    		this.atm = new at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.ATMcyclic<Inputtape,Worktape>(atmStates, atmWork, (byte)0);
        }
        return atm.compute(atmInput);
    }

	public at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.ComputationTree<Worktape> getComputationTree(){
		at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.ComputationTree<Worktape> cT=this.atm.getComputationTree();
		cT.setStateIdToName((byte)1, "catTurn");
		cT.setNameToStateId("catTurn",(byte) 1);
		cT.setStateIdToName((byte)0, "mouseTurn");
		cT.setNameToStateId("mouseTurn",(byte) 0);

		return cT;
	}
}
