package at.ac.tuwien.dbai.staff.dvorak.alternation.examples;
/** Lexicographically First Maximal Clique **/

public class LFMC {
	private at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.ATM<Inputtape,Worktape> atm;

	public class Inputtape {
		UndirectedGraph<Integer> graph;
		UndirectedGraph.Node<Integer> vertex;

		private Inputtape(UndirectedGraph<Integer> graph,UndirectedGraph.Node<Integer> vertex) {
			super();
			this.graph=graph;
			this.vertex=vertex;
		}
	}

    public class Worktape implements at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.InterfaceWorktape<Inputtape> {
    	UndirectedGraph.Node<Integer> node;

    	private Worktape(){
    		super();
    	}

    	private Worktape(Worktape worktape) {
    		this.node = worktape.node;
    	}

    	public void reset(Inputtape atmInputtape) {

    		this.node=null;
    		this.node = atmInputtape.vertex;
    	}

    	public Worktape clone(){
    		return new Worktape(this);
    	};

    	public boolean equals( Object o ) {
        	if ( o == null ) return false;
            if (! o.getClass().equals(getClass()) ) return false;
            Worktape tape2=(Worktape) o;
    		if(this.node==null){
    			if(!(tape2.node==null)) return false;
    		} else {
    		  	if(!this.node.equals(tape2.node)) return false;
    		}
            return true;
    	}

    	@Override
    	public int hashCode() {
    		final int prime = 31;
    		int result = 1;


    		result = prime * result + ((node == null) ? 0 : node.hashCode());

    		return result;
    	}

    	public String toString() {
    		StringBuffer ausgabe= new StringBuffer();

    		if (node==null) {
    			ausgabe.append("node=null; ");
    		} else {
    			ausgabe.append("node=("+((Object)node).toString()+"); ");
    		}

    		return ausgabe.toString();
    	}
    }

    private class inClique extends at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.State<Inputtape,Worktape>  {
    	
    	public at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.ResultTuple<Worktape> compute(Worktape atmWorktape) {
    		at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.ResultTuple<Worktape> atmResultTuple;
    		Worktape atmWorktape2;
    		if (atmWorktape.node.getObject().equals(1)){
    					return at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.ResultTuple.ACCEPT;
    				}
    				atmResultTuple = new at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.ResultTuple<Worktape>(false, false, true);
    		atmWorktape2=atmWorktape.clone();
    		for (int i=1;i<atmWorktape.node.getObject();i++) {
    						if(!atmInputtape.graph.get(i).getNeighbours().contains(atmWorktape.node)) {
    							{
    								atmWorktape.node=atmInputtape.graph.get(i);
    							}
    		atmResultTuple.add((byte)1,atmWorktape);
    		atmWorktape=atmWorktape2.clone();
    						}
    					}
    		return atmResultTuple;
    	}
    }

    private class notInClique extends at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.State<Inputtape,Worktape>  {
    	
    	public at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.ResultTuple<Worktape> compute(Worktape atmWorktape) {
    		at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.ResultTuple<Worktape> atmResultTuple;
    		Worktape atmWorktape2;
    		if (atmWorktape.node.getObject().equals(1)){
    					return at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.ResultTuple.REJECT;
    				}
    				atmResultTuple = new at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.ResultTuple<Worktape>(false, false, false);
    		atmWorktape2=atmWorktape.clone();
    		for (int i=1;i<atmWorktape.node.getObject();i++) {
    						if(!atmInputtape.graph.get(i).getNeighbours().contains(atmWorktape.node)) {
    							{
    								atmWorktape.node=atmInputtape.graph.get(i);
    							}
    		atmResultTuple.add((byte)0,atmWorktape);
    		atmWorktape=atmWorktape2.clone();
    						}
    					}
    		return atmResultTuple;
    	}
    }

    public boolean compute(UndirectedGraph<Integer> graph,UndirectedGraph.Node<Integer> vertex) {

    	Inputtape atmInput = new Inputtape(graph,vertex);
    	if (this.atm==null) {
    		Worktape atmWork = new  Worktape();
    		java.util.Map<Byte,at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.State<Inputtape,Worktape>> atmStates = new java.util.HashMap<Byte,at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.State<Inputtape,Worktape>>();

    		atmStates.put((byte)1,new notInClique());
    		atmStates.put((byte)0,new inClique());

    		this.atm = new at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.ATMacyclic<Inputtape,Worktape>(atmStates, atmWork, (byte)0);
        }
        return atm.compute(atmInput);
    }

	public at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.ComputationTree<Worktape> getComputationTree(){
		at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.ComputationTree<Worktape> cT=this.atm.getComputationTree();
		cT.setStateIdToName((byte)1, "notInClique");
		cT.setNameToStateId("notInClique",(byte) 1);
		cT.setStateIdToName((byte)0, "inClique");
		cT.setNameToStateId("inClique",(byte) 0);

		return cT;
	}
}