package at.ac.tuwien.dbai.staff.dvorak.alternation.examples;
import java.util.List;
import java.util.Set;
import java.util.Map;


public class OneDimensionalCellularAutomata {
	private at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.ATM<Inputtape,Worktape> atm;

	public class Inputtape {
		char[] cells;
		int cell;
		char finalstate;
		int T;
		Byte r;
		Map<List<Character>,Character> rphi;

		private Inputtape(char[] cells,int cell,char finalstate,int T,Byte r,Map<List<Character>,Character> rphi) {
			super();
			this.cells=cells;
			this.cell=cell;
			this.finalstate=finalstate;
			this.T=T;
			this.r=r;
			this.rphi=rphi;
		}
	}

    public class Worktape implements at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.InterfaceWorktape<Inputtape> {
    	char caState;
    	List<Character> parents;
    	int position;
    	int t;

    	private Worktape(){
    		super();
    	}

    	private Worktape(Worktape worktape) {
    		this.caState = worktape.caState;
    		this.position = worktape.position;
    		this.t = worktape.t;
    		this.parents = worktape.parents;
    	}

    	public void reset(Inputtape atmInputtape) {

    		this.parents=null;
    		this.t = atmInputtape.T;
    		this.position = atmInputtape.cell;
    		this.caState = atmInputtape.finalstate;
    	}

    	public Worktape clone(){
    		return new Worktape(this);
    	};

    	public boolean equals( Object o ) {
        	if ( o == null ) return false;
            if (! o.getClass().equals(getClass()) ) return false;
            Worktape tape2=(Worktape) o;
    		if(this.parents==null){
    			if(!(tape2.parents==null)) return false;
    		} else {
    		  	if(!this.parents.equals(tape2.parents)) return false;
    		}
    		if(this.caState!=tape2.caState) return false;
    		if(this.position!=tape2.position) return false;
    		if(this.t!=tape2.t) return false;
            return true;
    	}

    	@Override
    	public int hashCode() {
    		final int prime = 31;
    		int result = 1;


    		result = prime * result + ((parents == null) ? 0 : parents.hashCode());
    		result = prime * result +  ((Object)caState).hashCode();
    		result = prime * result +  ((Object)position).hashCode();
    		result = prime * result +  ((Object)t).hashCode();

    		return result;
    	}

    	public String toString() {
    		StringBuffer ausgabe= new StringBuffer();

    		if (parents==null) {
    			ausgabe.append("parents=null; ");
    		} else {
    			ausgabe.append("parents=("+((Object)parents).toString()+"); ");
    		}

    		 ausgabe.append("caState=(" +((Object)caState).toString()+"); ");
    		 ausgabe.append("position=(" +((Object)position).toString()+"); ");
    		 ausgabe.append("t=(" +((Object)t).toString()+"); ");
    		return ausgabe.toString();
    	}
    }

    private class ExistsState extends at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.State<Inputtape,Worktape>  {
    	
    	public at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.ResultTuple<Worktape> compute(Worktape atmWorktape) {
    		at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.ResultTuple<Worktape> atmResultTuple;
    		Worktape atmWorktape2;
    		if (atmWorktape.t==0) {
    					if (atmWorktape.position>=0 && atmWorktape.position<atmInputtape.cells.length) {
    						if (atmWorktape.caState==atmInputtape.cells[atmWorktape.position]){
    							return at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.ResultTuple.ACCEPT;
    						} else {
    							return at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.ResultTuple.REJECT;
    						}
    					} else {
    						if (atmWorktape.caState=='b'){
    							return at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.ResultTuple.ACCEPT;
    						} else {
    							return at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.ResultTuple.REJECT;
    						}
    					}
    				}
    				atmResultTuple = new at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.ResultTuple<Worktape>(false, false, false);
    		atmWorktape2=atmWorktape.clone();
    		for (Map.Entry<List<Character>,Character> hilf : atmInputtape.rphi.entrySet()){
    						if(hilf.getValue().equals(atmWorktape.caState)) {
    							{
    								atmWorktape.parents=hilf.getKey();
    							}
    		atmResultTuple.add((byte)1,atmWorktape);
    		atmWorktape=atmWorktape2.clone();
    						}
    					}
    		return atmResultTuple;
    	}
    }

    private class ForallState extends at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.State<Inputtape,Worktape>  {
    	
    	public at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.ResultTuple<Worktape> compute(Worktape atmWorktape) {
    		at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.ResultTuple<Worktape> atmResultTuple;
    		Worktape atmWorktape2;
    		atmWorktape.t--;
    				atmResultTuple = new at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.ResultTuple<Worktape>(false, false, true);
    		atmWorktape2=atmWorktape.clone();
    		for (int i=0 ; i<=2*atmInputtape.r; i++){
    						{
    							atmWorktape.position=atmWorktape.position-atmInputtape.r+i;
    							atmWorktape.caState=atmWorktape.parents.get(i);
    						}
    		atmResultTuple.add((byte)0,atmWorktape);
    		atmWorktape=atmWorktape2.clone();
    					}
    		return atmResultTuple;
    	}
    }

    public boolean compute(char[] cells,int cell,char finalstate,int T,Byte r,Map<List<Character>,Character> rphi) {

    	Inputtape atmInput = new Inputtape(cells,cell,finalstate,T,r,rphi);
    	if (this.atm==null) {
    		Worktape atmWork = new  Worktape();
    		java.util.Map<Byte,at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.State<Inputtape,Worktape>> atmStates = new java.util.HashMap<Byte,at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.State<Inputtape,Worktape>>();

    		atmStates.put((byte)0,new ExistsState());
    		atmStates.put((byte)1,new ForallState());

    		this.atm = new at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.ATMacyclic<Inputtape,Worktape>(atmStates, atmWork, (byte)0);
        }
        return atm.compute(atmInput);
    }

	public at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.ComputationTree<Worktape> getComputationTree(){
		at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.ComputationTree<Worktape> cT=this.atm.getComputationTree();
		cT.setStateIdToName((byte)0, "ExistsState");
		cT.setNameToStateId("ExistsState",(byte) 0);
		cT.setStateIdToName((byte)1, "ForallState");
		cT.setNameToStateId("ForallState",(byte) 1);

		return cT;
	}
}