package at.ac.tuwien.dbai.staff.dvorak.alternation.runtime;

import java.util.Map;

/**
 * This class simulates an alternating machine. (without tabled evaluation)
 *
 * @author Wolfgang Dvorak
 *
 * @param <GInputtape> Container class for the input variables
 * @param <GWorktape> Container class for the work variables
 */
public class ATMsimple<GInputtape,GWorktape extends InterfaceWorktape<GInputtape>> implements ATM<GInputtape, GWorktape>{

	private Map<Byte,State<GInputtape,GWorktape>> states;
	private byte startState;
	private GWorktape worktape;

	/**
	 * 
	 * @param states algorithms states
	 * @param worktape an empty worktape
	 * @param startState stateid from the state the computation should start with
	 */
	public ATMsimple(Map<Byte,State<GInputtape,GWorktape>> states,GWorktape worktape, byte startState){
		this.states=states;
		this.startState=startState;
		this.worktape=worktape;
	}
	/* (non-Javadoc)
	 * @see at.ac.tuwien.dbai.staff.dvorak.atm.ATM#compute(GInputtape)
	 */
	public boolean compute(GInputtape input){
		byte stateId=startState;

		for(State<GInputtape,GWorktape> state: states.values()){
			state.setInput(input);
		}
		worktape.reset(input);

		ComputationNode<GWorktape> rootNode = new ComputationNode<GWorktape>(stateId,worktape);

		return computeNode(rootNode);
	};

	/**
	 * computes the result of a ComputationNode
	 * @param node ComputationNode
	 * @return
	 */
	private boolean computeNode(ComputationNode<GWorktape> node) {
		ComputationNode<GWorktape> node2;

		ResultTuple<GWorktape> result;
		GWorktape worktape=(GWorktape)node.worktape.clone();

		result=states.get(node.state).compute(worktape);

		if (result.accept){
			return true;
		}

		if (result.reject){
			return false;
		}

		if (result.forall){
			/*
			 * What to do if there are no successors ? - default behaviour is accepting
			 * if (result.successors.isEmpty() ){return false;}
			 */
			node.setQuantifier(true);
			for (ResultTuple.AtmConfiguration<GWorktape> work : result.successors) {
				node2=new ComputationNode<GWorktape>(work.state,work.worktape);
				if (!computeNode(node2)){
					return false;
				}
			}
			return true;
		} else {
			node.setQuantifier(false);
			for (ResultTuple.AtmConfiguration<GWorktape> work : result.successors) {
				node2=new ComputationNode<GWorktape>(work.state,work.worktape);
				if (computeNode(node2)){
					return true;
				}
			}
			return false;
		}
	}

	/* (non-Javadoc)
	 * @see at.ac.tuwien.dbai.staff.dvorak.atm.ATM#getComputationTree()
	 */
	public ComputationTree<GWorktape> getComputationTree() {
		return null;
	}
}